#
# QuickLinks Zope Management interface enhancement.
# Peter Bengtsson, <mail@peterbe.com>
# 2002, ZPL license
#
#

from Globals import DTMLFile, InitializeClass
from OFS.SimpleItem import Item
from Acquisition import Implicit
from Persistence import Persistent
from DateTime import DateTime

import meta_type_icons

class QuickLinks(Item, Implicit, Persistent):
    """ """
    id = 'QuickLinks'
    title = meta_type = 'Quick Links'
    manage_options = ({'label': 'Configure', 'action':'configureQuickLinksForm'},)

    configureQuickLinksForm = DTMLFile('dtml/configureForm', globals())

    def writeCookie(self, value, response):
        """ write response cookie """
        then = self.ZopeTime()+300
        self.REQUEST.set('tmp_ql_meta_types',value)
        response.setCookie('ql_meta_types', value,
                           path='/', expires=then.rfc822())
        
    def configureQuickLinks(self, meta_types=[], REQUEST=None):
        """ """
        request = self.REQUEST
        response = request.RESPONSE
        self.writeCookie(meta_types, response)

        if REQUEST is not None:
            return self.configureQuickLinksForm(self, REQUEST)

    def make_ql_property(self):
        """ """
        request = self.REQUEST
        if not request.cookies.has_key('ql_meta_types'):
            self.writeCookie([], request.RESPONSE)

    def getAllMetaTypes(self):
        """ """
        return self.filtered_meta_types()

    def addMetaTypes(self, all_meta_types=[], REQUEST=None):
        """ """
        if type(all_meta_types)==type('s'):
            all_meta_types=[all_meta_types]
        ql_meta_types = self.get_ql_meta_types()
        for meta_type in all_meta_types:
            if meta_type not in ql_meta_types:
                ql_meta_types.append(meta_type)
        self.writeCookie(ql_meta_types, self.REQUEST.RESPONSE)

        if REQUEST is not None:
            return self.configureQuickLinksForm(self, self.REQUEST)

    def removeMetaTypes(self, ql_meta_types=[], REQUEST=None):
        """ """
        new_ql_meta_types= []
        for meta_type in self.get_ql_meta_types():
            if meta_type not in ql_meta_types:
                new_ql_meta_types.append(meta_type)

        self.writeCookie(new_ql_meta_types, self.REQUEST.RESPONSE)

        if REQUEST is not None:
            return self.configureQuickLinksForm(self, self.REQUEST)

    def _rand(self):
        """ return random string """
        return str(int(DateTime()))
    
    def get_ql_meta_types(self):
        """ """
        request = self.REQUEST
        if request.has_key('tmp_ql_meta_types'):
            return request['tmp_ql_meta_types']
        else:
            s=request.cookies['ql_meta_types']
            if s.startswith("[") and s.endswith("]"):
                exec "s= %s"%s
                return s
            else:
                return []
    
    def showQuickLinks(self, all_meta_types=None):
        """ """
        self.make_ql_property()
        if all_meta_types is None:
            all_meta_types = self.getAllMetaTypes()
        paths = {}
        for meta_type in all_meta_types:
            paths[meta_type['name']] = meta_type['action']

        s = ''
        icons_dict = meta_type_icons.dict
        for meta_type in self.get_ql_meta_types():
            if paths.has_key(meta_type):
                if icons_dict.has_key(meta_type):
                    mod_meta_type = '<img src="/%s" alt="%s" border="0" />'%\
                                    (icons_dict[meta_type], meta_type)
                else:
                    mod_meta_type=meta_type
                s = """%s
                <a href="%s" title="%s"
                style="font-size:10px;text-decoration:underline;">%s</a>
                &nbsp;&nbsp;"""%(s,
                                 paths[meta_type],
                                 meta_type,
                                 mod_meta_type
                                 )
        return s


    def showMetaTypeIcons(self, meta_types=[], joiner=''):
        """ By a list of meta types, return their icons """
        if type(meta_types)==type('s'):
            meta_types=[meta_types]
        #all_meta_types = self.getAllMetaTypes()
        paths=[]
        icons_dict = meta_type_icons.dict
        for meta_type in meta_types:
            if icons_dict.has_key(meta_type):
                tag='<img src="/%s" alt="%s" border="0" />'%\
                     (icons_dict[meta_type], meta_type)
                paths.append(tag)
            else:
                paths.append(meta_type)
        return joiner.join(paths)

    def getTypicalSelection(self, which=None):
        """ Return some typical selections.
            If which is None return all alternatives.
        """
        typical={}
        typical[1] = ['Folder','DTML Method','Script (Python)','Image',
                     'Page Template','Z SQL Method']
        typical[2] = ['Folder','DTML Document','DTML Method','Image']
        typical[3] = ['Folder','DTML Method','Script (Python)','Page Template',
                      'Image','File','External Method','Z SQL Method',
                      'User Folder']

        if which is None:
            return [typical[1],typical[2],typical[3]]
        else:
            try:
                which=int(which)
            except ValueError:
                return None
            try:
                return typical[which]
            except KeyError:
                return None

    def getTypicalSelections(self):
        """ shortcut for tidy syntax """
        return self.getTypicalSelection()

    def addTypicalSelection(self, typical, REQUEST=None):
        """ Use a typical and add it """
        # remove old
        old_selection = self.get_ql_meta_types()
        self.removeMetaTypes(old_selection)
        # add new
        typical = int(typical)
        typical_selection = self.getTypicalSelection(typical)
        return self.addMetaTypes(typical_selection, REQUEST)
                     
        


InitializeClass(QuickLinks)


